@@ -25,9 +25,9 @@ package Sidef::Module::OO {
             ? (
                map {
                    local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                       ref($_) eq __PACKAGE__            ? $_->{module}
-                     : ref($_) eq 'Sidef::Variable::Ref' ? $_->get_var->get_value
+                   ref($_) eq __PACKAGE__ ? $_->{module}
                      : index(ref($_), 'Sidef::') == 0 ? $_->get_value
+                     : ref($_) eq 'REF' ? ${$_}
                      : $_
                  } @arg
               )
@@ -40,7 +40,7 @@ package Sidef::Module::OO {
         };
 
         if (@results > 1) {
-            return Sidef::Types::Array::List->new(map { Sidef::Perl::Perl->to_sidef($_) } @results);
+            return (map { Sidef::Perl::Perl->to_sidef($_) } @results);
         }
 
         Sidef::Perl::Perl->to_sidef($results[0]);
